package com.cs.camerasecure;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Array;
import java.security.Provider;
import java.security.Security;
import java.util.ArrayList;
import java.util.Arrays;

import com.cs.camerasecure.PreferencesActivity.ValidPass;
import com.cs.camerasecure.util.ImageUtils;
import com.cs.camerasecure.util.OptionsUtils;
import com.cs.camerasecure.util.SystemUiHider;
import com.cs.cryptographic.AESCript;
import com.cs.cryptographic.RC4Crypt;
import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory;
import android.hardware.Camera;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.ShutterCallback;
import android.hardware.Camera.Size;
import android.media.MediaScannerConnection;
import android.media.MediaScannerConnection.MediaScannerConnectionClient;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.webkit.MimeTypeMap;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

/**
 * An example full-screen activity that shows and hides the system UI (i.e.
 * status bar and navigation/system bar) with user interaction.
 * 
 * @see SystemUiHider
 */
public class MainActivity extends Activity implements SurfaceHolder.Callback, MediaScannerConnectionClient {
	private Camera camera;
	public ImageButton cameraClick;
	private SurfaceHolder mHolder; 
	private SurfaceView surfaceView = null;
	private MediaScannerConnection msc = null;
	private static final String FILE_TYPE="image/*";
	private int DEFAULT_WIDTH_PREVIEW = 400;
	private int DEFAULT_HEIGHT_PREVIEW = 300;

	final int SELECT_IMAGE = 1300;

	public final static String DEFAULT_IMAGE_FOLDER = File.separator +Environment.DIRECTORY_DCIM + File.separator + "Secure images";
	

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Log.d(MainActivity.class.getName(), "Init. onCreate");
		super.onCreate(savedInstanceState);
		
		// Oculta barra de titulo. Fija la pantalla a horizontal.
		this.setRequestedOrientation(
				 ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
		        WindowManager.LayoutParams.FLAG_FULLSCREEN);
		
		setContentView(R.layout.activity_main);

		if(PreferencesActivity.readAndConfigPreferences(this) != ValidPass.True)
		{
			showPreferencesDialog();
		}
		
		this.surfaceView = (SurfaceView)findViewById(R.id.surfaceView1);
		mHolder = surfaceView.getHolder();
		mHolder.addCallback(this);
		mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);  

		cameraClick = (ImageButton) findViewById(R.id.imageButton);
		cameraClick.setOnClickListener( new OnClickListener() {			
			public void onClick(View v)
			{
				camera.takePicture(shutterCallback, rawCallback,jpegCallback);
			}
		});  			
		Log.d(MainActivity.class.getName(), "End. onCreate");
	}
	
	protected void onDestroy()
	{
		Log.d(MainActivity.class.getName(), "Init. onDestroy");
		super.onDestroy();
		OptionsUtils.getInstance().clearMediumPass();
		Log.d(MainActivity.class.getName(), "End. onDestroy");
	}
	
//	@Override
//	protected void onPause()
//	{
//		Log.d(MainActivity.class.getName(), "Init. onPause");
//		super.onPause();
//		
//		Log.d(MainActivity.class.getName(), "End. onPause");
//	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event)  {
		try
		{
			Log.d(MainActivity.class.getName(), "Init. onKeyDown");
			if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0) {
				camera.stopPreview();
				//camera.release();
				//camera = null; 
				//return false;
			}
			Log.d(MainActivity.class.getName(), "End. onKeyDown");
		} catch(Exception ex)
		{
			Log.d(MainActivity.class.getName(), "Error. onKeyDown: " + ex.getMessage());
		}
		return super.onKeyDown(keyCode, event);
	}

	ShutterCallback shutterCallback = new ShutterCallback()
	{		
		public void onShutter()
		{   
			Toast.makeText(getApplication(), getApplication().getText(R.string.txtWaitSaving), Toast.LENGTH_LONG).show();
		}
	};

	/** Handles data for raw picture */
	PictureCallback rawCallback = new PictureCallback()
	{
		public void onPictureTaken(byte[] data, Camera camera)
		{   
			//Toast.makeText(getApplication(), getApplication().getText(R.string.txtWaitSaving), Toast.LENGTH_LONG).show();
			Log.d(MainActivity.class.getName(), "onPictureTaken rawCallback");
		}
	}; 

	/** Handles data for jpeg picture */
	PictureCallback jpegCallback = new PictureCallback()
	{
		public void onPictureTaken(byte[] data, Camera camera)
		{
			
			Log.d(MainActivity.class.getName(), "Init. onPictureTaken");
			try {
		
				for (Provider provider: Security.getProviders()) {
					  System.out.println(provider.getName());
					  for (String key: provider.stringPropertyNames())
					    System.out.println("\t" + key + "\t" + provider.getProperty(key));
				}
				// Obtenemos el nombre de la imagen a grabar.
				String filename = ImageUtils.buildFileName();
				// Nos aseguramos de crear el direcorio donde se guardarn las imgenes.
				File storagePath = new File(Environment.getExternalStorageDirectory() + MainActivity.DEFAULT_IMAGE_FOLDER); 
				storagePath.mkdirs();
				// Test de memoria disponible.
				//				Debug.MemoryInfo memoryInfo = new Debug.MemoryInfo();
				//				Debug.getMemoryInfo(memoryInfo);

				// Convertimos la imagen original en un bmp.
				Bitmap bmpOriginal = BitmapFactory.decodeByteArray(data, 0, data.length);
				// Escalamos la imagen a la resolucin dada por la configuracin.
				//Bitmap bmpDecoded1 = Bitmap.createScaledBitmap(bmpOriginal, instance.width, instance.height, false);
				Bitmap bmpEncoded = Bitmap.createScaledBitmap(bmpOriginal, OptionsUtils.getInstance().getWidth(), OptionsUtils.getInstance().getHeight(), false);
				bmpOriginal.recycle();
				Bitmap bmpResized = null;
				if(OptionsUtils.getInstance().getPreview())
				{
					bmpResized =  Bitmap.createScaledBitmap(bmpEncoded, DEFAULT_WIDTH_PREVIEW, DEFAULT_HEIGHT_PREVIEW, false);
							//bmpEncoded.copy(Config.ARGB_8888, false);
				}

				// Guardamos la imagen escalada.
				String file1Path = storagePath + File.separator + filename;
				byte[] pixelsData = ImageUtils.readPixels(bmpEncoded);
				int len = pixelsData.length;
				Log.i(MainActivity.class.getName(), "SecurityLevel is: " + OptionsUtils.getInstance().getSecurityLevel().toString());
				Log.i(MainActivity.class.getName(), "Encrypded password is: " + OptionsUtils.getInstance().getEncryptedPass());
				Log.i(MainActivity.class.getName(), "Encrypting with password: " + OptionsUtils.getInstance().getPass());
				pixelsData = RC4Crypt.getInstance().encrypt(OptionsUtils.getInstance().getPass().getBytes("UTF-8"), pixelsData);
//				pixelsData = AESCript.encrypt(OptionsUtils.getInstance().getPass(), pixelsData);
//				byte []dataTemp = null;
//				if(len != pixelsData.length)
//				{
//					bmpEncoded = Bitmap.createScaledBitmap(bmpEncoded, OptionsUtils.getInstance().getWidth(), OptionsUtils.getInstance().getHeight() + 1, false);
//					dataTemp = new byte[OptionsUtils.getInstance().getWidth() * (OptionsUtils.getInstance().getHeight() + 1) * 4];
//					System.arraycopy(pixelsData, 0, dataTemp, 0, pixelsData.length);
//					pixelsData = dataTemp;
//				}

				ImageUtils.putPixels(bmpEncoded, pixelsData);
//				int [] iPixels = new int[bmpEncoded.getWidth()* bmpEncoded.getHeight()];
//				bmpEncoded.getPixels(iPixels, 0, bmpEncoded.getWidth(), 0, 0, bmpEncoded.getWidth(), bmpEncoded.getHeight());
//				byte [] bPixel = ImageUtils.readPixels(bmpEncoded);
//				byte [] aux = new byte[64];
//				System.arraycopy(pixelsData, 480000, aux, 0, 64);
//				ImageUtils.printBytes(aux, 64);
				ImageUtils.saveBitmap(bmpEncoded, file1Path);
				
				
				
				
				bmpEncoded.recycle();

				if(OptionsUtils.getInstance().getPreview())
				{
					Intent i = new Intent(getApplication(), PreviewActivity.class);
					i.putExtra("Bitmap", bmpResized);
					Log.d(MainActivity.class.getName(), "Imagen guardada: " + file1Path);
					startActivity(i);
				} else
				{
					Toast.makeText(getApplication(), getApplication().getText(R.string.txtPictureSaved) + file1Path, Toast.LENGTH_LONG).show();
					camera.startPreview();
				}

			} catch (Exception ex) {
				String msg = ex != null ? ex.getMessage() : getApplication().getText(R.string.txtUnknowError).toString();
				Toast.makeText(getApplication(), getApplication().getText(R.string.txtErrorSaving) + msg, Toast.LENGTH_LONG).show();
				Log.d(MainActivity.class.getName(), "Error. onPictureTaken." + msg);
				camera.startPreview();
			}
			Log.d(MainActivity.class.getName(), "End. onPictureTaken");
		}
	};

//	http://stackoverflow.com/questions/5540981/picture-distorted-with-camera-and-getoptimalpreviewsize
	private Camera.Size getBestSize(ArrayList<Size> sizelist, int width, int height)
	{
	        Camera.Size result=null;    
	        int betterScore = 10000;
	        
	        for (Camera.Size size : sizelist) 
	        {
	        	int score = Math.abs(size.width - width);
	        	if(betterScore > score)
	        	{
	        		betterScore = score;
	        		result = size;
	        	}
	            if (size.width<=width && size.height<=height) {
	                if (result==null) {
	                    result=size;
	                } else {
	                    int resultArea=result.width*result.height;
	                    int newArea=size.width*size.height;

	                    if (newArea>resultArea) {
	                        result=size;
	                    }
	                }
	            }
	        }
	    return result;

	}
	
	@Override
	public void surfaceChanged(SurfaceHolder arg0, int arg1, int arg2,
			int arg3) {
		// Set camera preview size,orientation,rotation using parameters 
		Log.d(MainActivity.class.getName(), "Init. surfaceChanged");
		try
		{
			Camera.Parameters parameters = camera.getParameters();
			///////////
			ArrayList<Size> lSize = (ArrayList<Size>) parameters.getSupportedPictureSizes();
			ArrayList<Size> lPreSize = (ArrayList<Size>) parameters.getSupportedPreviewSizes();
			Size optimalSize = getBestSize(lSize, OptionsUtils.getInstance().getHeight(), OptionsUtils.getInstance().getHeight());
			Size optimalPreSize = getBestSize(lPreSize, OptionsUtils.getInstance().getHeight(), OptionsUtils.getInstance().getHeight());
			parameters.setPreviewSize(optimalPreSize.width, optimalPreSize.height);
			parameters.setPictureSize(optimalSize.width, optimalSize.height);
			parameters.setFlashMode(Camera.Parameters.FLASH_MODE_ON);
			/////////////////////
			parameters.set("orientation", "portrait");
			parameters.setPictureSize(OptionsUtils.getInstance().getWidth(), OptionsUtils.getInstance().getHeight());
			camera.setParameters(parameters);
		} catch (Exception e) {
			Log.e(MainActivity.class.getName(), "Error: " + e.getMessage());
		} finally
		{
			try
			{
				camera.startPreview();
			} catch(Exception e)
			{
				Log.e(MainActivity.class.getName(), "Error startPreview: " + e.getMessage());
			}
		}						
		Log.d(MainActivity.class.getName(), "End. surfaceChanged");
	}

	@Override
	public void surfaceCreated(SurfaceHolder arg0) {
		Log.d(MainActivity.class.getName(), "Init. surfaceCreated");
		try {
			camera = Camera.open();
			camera.setPreviewDisplay(mHolder);
		}
		catch (IOException e)
		{
//			if(camera != null)
//			{
//				camera.stopPreview();
//				camera.release();
//			}
			Toast.makeText(getApplication(), "Error: " + e.getMessage(), Toast.LENGTH_LONG).show();
			finish();
		}
		Log.d(MainActivity.class.getName(), "End. surfaceCreated");
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder arg0) {
		Log.d(MainActivity.class.getName(), "Init. surfaceDestroyed");
		camera.stopPreview();
		camera.release();
		Log.d(MainActivity.class.getName(), "End. surfaceDestroyed");
	}

	//Funcin que inserta el men.
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		Log.d(MainActivity.class.getName(), "Init. onCreateOptionsMenu");
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.menu, menu);
		Log.d(MainActivity.class.getName(), "End. onCreateOptionsMenu");
		return true;
	}

	@Override 
	public boolean onOptionsItemSelected(MenuItem item) {
		Log.d(MainActivity.class.getName(), "Init. onOptionsItemSelected");
		switch (item.getItemId()) {
		case R.id.mnAbout:
			showAboutDialog();			
			break;
		case R.id.mnConfig:
			showPreferencesDialog();
			break;
		case R.id.mnOpenGalery:
			showGalleryDialog();
			break;
		}
		Log.d(MainActivity.class.getName(), "End. onOptionsItemSelected");
		return true;
	}

	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		Log.d(MainActivity.class.getName(), "Init. onActivityResult");
		super.onActivityResult(requestCode, resultCode, data);
		switch(requestCode) { 
		case SELECT_IMAGE:
			if(resultCode == RESULT_OK){  
				Uri selectedImage = data.getData();
				String[] filePathColumn = {MediaStore.Images.Media.DATA};

				Cursor cursor = getContentResolver().query(selectedImage, filePathColumn, null, null, null);
				cursor.moveToFirst();

				int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
				String filePath = cursor.getString(columnIndex);
				cursor.close();
				if(filePath.endsWith(".png"))
				{
					//Bitmap picture = BitmapFactory.decodeFile(filePath);
	
					Intent i = new Intent(this, GalleryActivity.class);
					i.setType(FILE_TYPE);
					i.putExtra("Bitmap",filePath);	
					//i.putExtra((String) getResources().getText(R.string.preferenceKeyPass),OptionsUtils.getInstance().getPass());
					startActivity(i);
				} else
				{
					Toast.makeText(getApplicationContext(), getApplication().getText(R.string.txtWrongImageFile), Toast.LENGTH_LONG).show();
				}
			}
		}
		Log.d(MainActivity.class.getName(), "End. onActivityResult");
	}

	private void showPreferencesDialog()
	{
		Intent i = new Intent(this, PreferencesActivity.class);		
		startActivity(i);
	}

	private void showGalleryDialog()
	{
		//http://stackoverflow.com/questions/12652498/why-are-images-from-newly-created-folder-not-showing-up
		sendBroadcast(new Intent(Intent.ACTION_MEDIA_MOUNTED,
                Uri.parse("file://"
                        + Environment.getExternalStorageDirectory() + MainActivity.DEFAULT_IMAGE_FOLDER)));
		Intent intent = new Intent(
                    Intent.ACTION_PICK,
                    android.provider.MediaStore.Images.Media.INTERNAL_CONTENT_URI);
		intent.setType(FILE_TYPE);
		startActivityForResult(Intent.createChooser(intent, getApplication().getText(R.string.txtSelectPicture)),SELECT_IMAGE);
	}

	private void showAboutDialog()
	{
		final Dialog dialog = new Dialog(this);
		dialog.setContentView(R.layout.about_layout);
		dialog.setTitle(getApplication().getText(R.string.app_name));

		TextView text = (TextView) dialog.findViewById(R.id.text);
		text.setText("http://www.camerasecure.net\nFrancisco Javier Fernndez Conde");
		ImageView image = (ImageView) dialog.findViewById(R.id.image);
		image.setImageResource(R.drawable.ic_launcher);
		dialog.show();
	}

	@Override
	public void onMediaScannerConnected() {
		 Log.d(MainActivity.class.getName(), "Init. onMediaScannerConnected");
		 this.msc.scanFile(DEFAULT_IMAGE_FOLDER, FILE_TYPE);  
		 Log.d(MainActivity.class.getName(), "End. onMediaScannerConnected");
	}

	@Override
	public void onScanCompleted(String path, Uri uri) {
		Log.d(MainActivity.class.getName(), "Init. onMediaScannerConnected");
		try {
			Log.d("onScanCompleted", uri + "success" + this.msc);
			Log.d(MainActivity.class.getName(), "URI " + uri);          
			if (uri != null) 
			{
				Intent intent = new Intent(Intent.ACTION_VIEW);
				intent.setData(uri);
				startActivity(intent);
			}
		} finally 
		{
			this.msc.disconnect();
			this.msc = null;
			
//			Intent intent = new Intent();
//			
//			intent.setType(FILE_TYPE);
//			intent.setAction(Intent.ACTION_GET_CONTENT);//
//			startActivityForResult(Intent.createChooser(intent, "Select Picture"),SELECT_IMAGE);
		}
	}
		
}
