package com.cs.camerasecure;


import java.util.Map;
import com.cs.camerasecure.util.OptionsUtils;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.widget.Toast;

public class PreferencesActivity extends PreferenceActivity{
	public enum ValidPass { True, False, Invalid };
	public final static int DEFAULT_WIDTH = 400;
	public final static int DEFAULT_HEIGHT = 300;
	public final static String PREFS_NAME = "preferencias";
	public final static String ENCRIPTED_PASS_KEY = "encryptedPassKey";
	private final static int MIN_PASS_LEN = 8;
	
	//Este constructor se llama automaticamente al comenzar una nueva actividad
	public PreferencesActivity()
	{ 	
	}
	
	@SuppressWarnings("deprecation")
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		getPreferenceManager().setSharedPreferencesName(PREFS_NAME);
		addPreferencesFromResource(R.xml.preferences);				
	}
	
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event)  {
		try
		{
			Log.d(PreferencesActivity.class.getName(), "Init. onKeyDown");
			if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0) {
				ValidPass validity = readAndConfigPreferences(getApplication());
				if(validity == ValidPass.False)
				{
					Toast.makeText(getApplication(), getApplication().getText(R.string.txtSetPass), Toast.LENGTH_LONG).show();
					return true;
				} if(validity == ValidPass.Invalid)
				{
					Toast.makeText(getApplication(), getApplication().getText(R.string.txtSetInvalidPass), Toast.LENGTH_LONG).show();
					return true;
				}else
				{
					 return super.onKeyDown(keyCode, event);
				}
		    }
		} catch(Exception ex)
		{
			Log.e(PreferencesActivity.class.getName(), "Error. onKeyDown: " + ex.getMessage());
		}
		Log.d(PreferencesActivity.class.getName(), "End. onKeyDown");
		return true;
	}
	
	public static void configResolutionPreference(Context myContext, SharedPreferences pm)
	{
		String resolutionKey = (String) myContext.getResources().getText(R.string.preferenceResolution);
		String resolution = (String) pm.getAll().get(resolutionKey);
		if(resolution == null)
		{
			// Si no hay resolucin configurada, se configura a la resolucin por defecto.
			OptionsUtils.getInstance().setWidth(DEFAULT_WIDTH);
			OptionsUtils.getInstance().setHeight(DEFAULT_HEIGHT);
			SharedPreferences.Editor editor = pm.edit();
			editor.putString(resolutionKey, DEFAULT_WIDTH + "x" + DEFAULT_HEIGHT);
			editor.commit();
		
		} else
		{
			Integer index = resolution.indexOf("x");
			OptionsUtils.getInstance().setWidth(Integer.parseInt(resolution.substring(0, index)));
			OptionsUtils.getInstance().setHeight(Integer.parseInt(resolution.substring(index + 1)));
		}
	}
	
	public static void configPreviewPreference(Context myContext, SharedPreferences pm)
	{
		String previewKey = (String) myContext.getResources().getText(R.string.preferenceKeyPreview);
		Boolean previewActive = (Boolean) pm.getAll().get(previewKey);
		if(previewActive == null)
		{
			// Si no hay resolucin configurada, se configura a la resolucin por defecto.
			OptionsUtils.getInstance().setPreview(true);
			SharedPreferences.Editor editor = pm.edit();
			editor.putBoolean(previewKey, OptionsUtils.getInstance().getPreview());
			editor.commit();
		
		} else
		{
			OptionsUtils.getInstance().setPreview(previewActive);
		}
	}
	
	private static void configLevelSecurity(Context myContext, SharedPreferences pm)
	{
		String levelSecurityKey = (String) myContext.getResources().getText(R.string.preferenceSecurityLevelKey);
		String levelSecurity = (String) pm.getAll().get(levelSecurityKey);
		if(levelSecurity == null)
		{
			OptionsUtils.getInstance().setSecurityLevel(OptionsUtils.SecurityLevel.Medium);
			SharedPreferences.Editor editor = pm.edit();
			editor.putString(levelSecurityKey, OptionsUtils.SecurityLevel.Medium.toString());
			editor.commit();
		} else
		{
			OptionsUtils.getInstance().setSecurityLevel(OptionsUtils.SecurityLevel.valueOf(levelSecurity));
		}
	}
	
	private static Boolean setPassWord(Context myContext, SharedPreferences pm, String passKey)
	{
		Boolean isValid = false;
		
		
		String password = (String) pm.getAll().get(passKey);
		String encryptedPassword = (String) pm.getAll().get(ENCRIPTED_PASS_KEY);
		encryptedPassword = encryptedPassword == null ? "" : encryptedPassword; //Para que nunca sea null y se pueda commparar
		switch(OptionsUtils.getInstance().getSecurityLevel())
		{
		case Low:
			{
				OptionsUtils.getInstance().setPass(password);			
				SharedPreferences.Editor editor = pm.edit();
				editor.putString(passKey, null);
				editor.putString(ENCRIPTED_PASS_KEY, OptionsUtils.getInstance().getEncryptedPass());			
				editor.commit();
				isValid = true;
			}
			break;
		case Medium:
			OptionsUtils.getInstance().setPass(password);
			SharedPreferences.Editor editor = pm.edit();
			// En nivel medio no se guardan contraseas
			editor.putString(passKey, null);
			if(encryptedPassword.length() == 0) //First time
			{														
				editor.putString(ENCRIPTED_PASS_KEY, OptionsUtils.getInstance().getEncryptedPass());					
				isValid = true;
			} else if(!OptionsUtils.getInstance().isPasswordOk(encryptedPassword, password))
			{
				Toast.makeText(myContext, myContext.getText(R.string.txtWrogPassword), Toast.LENGTH_LONG).show();
				isValid = false;
			} else 
			{
				isValid = true;
			}
			editor.commit();
			break;
//		case Hight:
//			break;
		}
		return isValid;
	}
	
	private static Boolean isPassWordValid(String pass)
	{
		Boolean ret = false;
		if(pass.length() >= MIN_PASS_LEN)
		{
			boolean upperFound = false;
			boolean lowerFound = false;
			boolean numericFound = false;
			for (char c : pass.toCharArray()) 
			{
			    if (Character.isUpperCase(c) || Character.isLowerCase(c))
			    {
			        upperFound = true;			     
			    	lowerFound = true;
			    } else if( Character.isDigit(c))
			    {
			    	numericFound = true;
			    }
			    if(upperFound && lowerFound && numericFound)
			    {
			    	ret = true;
			    	break;
			    }
			}
		}
		return ret;
	}
	
	private static ValidPass configPass(Context myContext, SharedPreferences pm) throws Exception
	{
		
		ValidPass isValid = ValidPass.False;
		String passKey = (String) myContext.getResources().getText(R.string.preferenceKeyPass);
		String password = (String) pm.getAll().get(passKey);
		
		//JFC Aqui se filtran los passwords no seguros.
		
		if(password != null)
		{
			if(isPassWordValid(password))
			{
				isValid = setPassWord(myContext,pm, passKey) ? ValidPass.True : ValidPass.False;
			} else
			{
				isValid = ValidPass.Invalid;
			}

		} else 
		{
			String encryptedPass = (String) pm.getAll().get(ENCRIPTED_PASS_KEY);
			if(encryptedPass != null)
			{
				switch(OptionsUtils.getInstance().getSecurityLevel())
				{
				case Low:
					String passRead = OptionsUtils.getInstance().decryptPass(encryptedPass);
					 if(OptionsUtils.getInstance().isPasswordOk(encryptedPass, passRead))
					 {
						 OptionsUtils.getInstance().setPass(passRead);
						 isValid = ValidPass.True;
					 }
					 break;
					
				case Medium:
					isValid = OptionsUtils.getInstance().getPass() != null ?  ValidPass.True : ValidPass.False;
					if(isValid == ValidPass.True)
					{
						SharedPreferences.Editor editor = pm.edit();
						editor.putString(passKey, null);
						editor.putString(ENCRIPTED_PASS_KEY, OptionsUtils.getInstance().getEncryptedPass());
						editor.commit();
					}
					break;
	//			case Hight:
	//				break;
				}
			} else
			{
				isValid =  ValidPass.False;
			}
			
		}
		return isValid;
	}
	
	public static ValidPass readAndConfigPreferences(Context myContext)
	{
		Log.d(PreferencesActivity.class.getName(), "Init. readPreferences");
		
		try
		{
			SharedPreferences pm = myContext.getSharedPreferences(
					PREFS_NAME, Context.MODE_PRIVATE);
			@SuppressWarnings("unused")
			Map<String, ?> map = pm.getAll();	
			
			configLevelSecurity(myContext, pm);
			configPreviewPreference(myContext, pm);
			ValidPass isValid = configPass(myContext, pm);
			configResolutionPreference(myContext, pm);
			if(isValid != ValidPass.True)
			{
				return isValid;
			}

		} catch (Exception e) {
			Log.e(PreferencesActivity.class.getName(), "Error. readPreferences." + e.getMessage());
			Toast.makeText(myContext, myContext.getText(R.string.txtErrorReadingPreferences), Toast.LENGTH_LONG).show();
		}
		Log.d(PreferencesActivity.class.getName(), "End. readPreferences");
		return OptionsUtils.getInstance().isValid();		
	}
}
