package com.cs.camerasecure.util;

import android.os.Build;
import android.util.Base64;
import android.util.Log;

import com.cs.camerasecure.PreferencesActivity.ValidPass;
import com.cs.cryptographic.RC4Crypt;
import com.cs.cryptographic.SHACypt;

public class OptionsUtils 
{
	public enum SecurityLevel {Low, Medium/*, Hight*/};
	private String pass = null;
	private Integer width;
	private Integer height;
	private SecurityLevel securityLevel;
	private Boolean preview;
	private String tempMediumPass = null;
	private static OptionsUtils instance = null;

	private OptionsUtils()
	{
		this.securityLevel = SecurityLevel.Medium;
	}

	public static OptionsUtils getInstance()
	{
		if(instance == null)
		{
			instance = new OptionsUtils();
		}
		return instance;
	}
	
	public Boolean getPreview() {
		return preview;
	}

	public void setPreview(Boolean preview) {
		this.preview = preview;
	}

	public String getEncryptedPass()
	{
		return this.pass;
	}
	
	public void clearMediumPass()
	{
		this.tempMediumPass = null;
	}

	public String getPass()
	{
		String password = null;
		try {
			byte [] data = null;
			switch(this.securityLevel)
			{
			case Low:					
				data = RC4Crypt.getInstance().decrypt(Build.DEVICE.getBytes("UTF-8") , Base64.decode(pass, Base64.DEFAULT));				
				password = new String(data);
				break;
			case Medium:
				password = this.tempMediumPass;
				break;
//			case Hight:			
//				break;
			}
		} catch (Exception e) {
			Log.e(OptionsUtils.class.getName(), "Error getPass: "+ e.getMessage());
		}
		return (password == null || password.length() == 0) ? null : password;
	}
	
	public Boolean isPasswordOk(String enctyped, String clear)
	{
		String myencrypted = "";
		byte[] data = null;
		try {
			switch(this.securityLevel)
			{
			case Low:									
				data = RC4Crypt.getInstance().encrypt(Build.DEVICE.getBytes("UTF-8") , clear.getBytes("UTF-8"));						
				myencrypted = Base64.encodeToString(data, Base64.DEFAULT);
				break;
			case Medium:
				myencrypted = SHACypt.computeHash(clear);
				break;
//			case Hight:			
//				break;
			}
		} catch (Exception e) {
			Log.e(OptionsUtils.class.getName(), "Error isPasswordOk: "+ e.getMessage());
		}		
		Log.i(OptionsUtils.class.getName(), "isPasswordOk: "+ myencrypted.equals(enctyped));
		return myencrypted.equals(enctyped);
	}
	
	public String decryptPass(String cryptedPass) throws Exception
	{
		String pass = null;
		if(this.securityLevel == SecurityLevel.Low)
		{
			byte [] data = RC4Crypt.getInstance().decrypt(Build.DEVICE.getBytes("UTF-8") , Base64.decode(cryptedPass, Base64.DEFAULT));
			pass = new String(data);
		}
		return pass;
	}

	public void setPass(String pass) 
	{		
		try {
			byte [] data = null;
			switch(this.securityLevel)
			{
			case Low:								
				data = RC4Crypt.getInstance().encrypt(Build.DEVICE.getBytes("UTF-8") , pass.getBytes("UTF-8"));	
				this.pass = Base64.encodeToString(data, Base64.DEFAULT);
				this.tempMediumPass = null;
				break;
			case Medium:
				this.tempMediumPass = pass;
				this.pass = SHACypt.computeHash(pass);
				break;
//			case Hight:	
//				this.tempMediumPass = null;
//				break;
			}
		} catch (Exception e) {
			Log.e(OptionsUtils.class.getName(), "Error setPass: "+ e.getMessage());
		}
		Log.i(OptionsUtils.class.getName(), "setPass: "+ getPass());
	}

	public Integer getWidth() {
		return width;
	}

	public void setWidth(Integer width) {
		this.width = width;
	}

	public Integer getHeight() {
		return height;
	}

	public void setHeight(Integer height) {
		this.height = height;
	}

	public SecurityLevel getSecurityLevel() {
		return securityLevel;
	}

	public void setSecurityLevel(SecurityLevel securityLevel)
	{
		String password = null;
		switch(this.securityLevel)
		{
		case Low:
//			if(securityLevel != SecurityLevel.Low)
//			{	
//				this.pass = null;
//			} 
			
			if(securityLevel == SecurityLevel.Medium)
			{
				password = getPass();
				this.securityLevel = securityLevel;
				setPass(password);
			}
			break;
		case Medium:
			if(securityLevel != SecurityLevel.Medium)
			{	
				this.pass = null;
			} 
			this.securityLevel = securityLevel;
			break;
//		case Hight:			
//			this.pass = null;
//			this.securityLevel = securityLevel;
//			break;
		}
		Log.i(OptionsUtils.class.getName(), "setSecurityLevel: "+ getSecurityLevel().toString());
	}

	public ValidPass isValid()
	{
		ValidPass valid = ValidPass.False;
		switch(this.securityLevel)
		{
		case Low:
			valid = getPass() == null ? ValidPass.False : ValidPass.True;
			break;
		case Medium:
			valid = this.tempMediumPass != null ? ValidPass.True : ValidPass.False;
			break;		
//		case Hight:			
//			valid = true;
//			break;
		}
		Log.i(OptionsUtils.class.getName(), "isOptionsPassValid: "+ valid.toString());
		return valid;
	}


}
